//
//  TemplatedString.swift
//  AppleLibs
//
//  Created by Tobias on 03.12.21.
//
//  Source: https://medium.com/flawless-app-stories/minimal-templating-system-in-swift-in-only-10-lines-b031963150e7
//

import Foundation

@dynamicMemberLookup
public struct TemplatedString
{
    public var template : String
    private var data : [String:String]
    public var evaluatedString : String { data.reduce(template) { $0.replacingOccurrences(of: "${#\($1.key)}", with: $1.value) } }
    
    public init(template: String, data: [String:String] = [:]) {
        self.template = template
        self.data = data
    }
    
    public subscript (dynamicMember member: String) -> CustomStringConvertible? {
        get {
            data[member]
        }
        set {
            data[member] = newValue?.description
        }
    }
    
    public subscript (dynamicMember member: String) -> Date {
        get {
            dateFormatter.date(from: data[member] ?? "") ?? Date(timeIntervalSince1970: 0)
        }
        set {
            data[member] = dateFormatter.string(from: newValue)
        }
    }
    
    public var dateFormatter : DateFormatter = {
        let formatter = DateFormatter()
        formatter.dateFormat = "yyyy-MM-dd'T'HH:mm:ssZZZZZ"
        formatter.calendar = Calendar(identifier: .gregorian)
        return formatter
    }()
}
